#!/usr/bin/env bash

#
# Script that configure wine and/or launch OpenGD77 CPS win32 binary
# Copyright (C) 2019-2020 - F1RMB, Daniel Caujolle-Bert <f1rmb.daniel@gmail.com>
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License.
# 
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.
# 

VERSION_STRING="0.0.1"

export WINEARCH="win32"
export WINEPREFIX=~/.wine-OpenGD77CPS

MIN_WINE_VERSION=4
WINE_BIN=$(command -v wine)
VERBOSE="yes"

CPS_URL="https://github.com/rogerclarkmelbourne/OpenGD77CPS/releases/download/R2020.08.01/OpenGD77CPSInstaller.exe"
CPS_BIN="$WINEPREFIX/drive_c/Program Files/OpenGD77CPS/OpenGD77CPS.exe"

WINETRICKS_BIN=$(command -v winetricks)
WINETRICKS_MODULES=("dotnet46")
WINETRICKS_CHECKED="no"
WINETRICKS_AVAILABLE="no"

##
##
##
function Usage() {
    echo "`basename -s .sh $0` launcher v$VERSION_STRING"
    echo ""
    echo "Usage:"
    echo "        `basename $0` [OPTION | /path/to/GD77CPS.exe]"
    echo ""
    echo "    Option can be:"
    echo "        -h, --help                Display this help."
    echo "        -p, --prepare             Prepare a clean wine configuration in order to run OpenGD77 CPS."
    echo "        -i, --install             Prepare a clean installation, download and install OpenGD77 CPS."
    echo "        -m, --install-modules     Install winetricks modules."
    echo "        -d, --download            Download and install the OpenGD77 CPS."
    echo "        -w, --winetricks          Starts winetricks."
    echo "        -c, --check               Check current wine's installation."
    echo ""
}

##
##
##
function printout() {
    if [ x"$VERBOSE" == "xyes" ]; then
	echo $*
    fi
}

##
##
##
function CheckWineVersion() {
   
    ## Check for wine binary
    if ! [ -x "$WINE_BIN" ]; then
	printout "Wine is missing, see https://wiki.winehq.org/Download to install it)."
	return 1
    else
	printout "Wine binary found: $WINE_BIN"
    fi

    mversion=$(wine --version | sed -e 's/wine\-//g' | awk 'BEGIN { FS="." } { print $1 }')

    if [ "$mversion" -lt $MIN_WINE_VERSION ]; then
	printout "Your Wine installed version should be updated to >= $MIN_WINE_VERSION."
	return 1
    else
	printout "Your Wine major version is $mversion, this is good."
    fi

    return 0
}

##
##
##
function CheckWinetricks() {
    ## Check for winetricks binary
    if [ $WINETRICKS_CHECKED = "no" ]; then
	if ! [ -x "$WINETRICKS_BIN" ]; then
	    printout "winetricks is missing, you should install it (e.g: sudo apt-get install winetricks)"
	    return 1
	fi
	export WINETRICKS_AVAILABLE="yes"
	export WINETRICKS_CHECKED="yes"
	printout "winetrick binary found: $WINETRICKS_BIN"
    else
	if [ $WINETRICKS_AVAILABLE != "no" ]; then
	    return 1
	fi
    fi
    
    return 0
}

##
##
##
function InstallWinetricksModules() {

    if [ $WINETRICKS_CHECKED = "no" ]; then
	CheckWinetricks
    fi

    if [ $WINETRICKS_AVAILABLE = "yes" ]; then
	winetricksModules=$($WINETRICKS_BIN list-installed)
	let offset=0
        let fCount=${#WINETRICKS_MODULES[@]}
	let n=0
	
	for n in $(seq 0 $(($fCount - 1))); do
	    ## Install module only if it's missing from current installation
	    if [ -z "$(echo $winetricksModules | grep ${WINETRICKS_MODULES[offset]})" ]; then
		echo " - Installing module '${WINETRICKS_MODULES[offset]}'"
		$WINETRICKS_BIN "${WINETRICKS_MODULES[offset]}"
		ret=$?
		if [ $ret != 0 ]; then
		    echo -ne "\n\nWinetricks failed to install module ${WINETRICKS_MODULES[offset]}.\n\n"
		    exit 1
		fi
	    fi
	    ((offset++))
	done
    else
	return 1
    fi

    return 0
}

##
##
##
function CheckWinetricksModules() {
    winetricksModules=$($WINETRICKS_BIN list-installed)

    let offset=0
    let fCount=${#WINETRICKS_MODULES[@]}
    let n=0
    
    for n in $(seq 0 $(($fCount - 1))); do
	if [ -z "$(echo $winetricksModules | grep ${WINETRICKS_MODULES[offset]})" ]; then
	    echo "Winetricks module '${WINETRICKS_MODULES[offset]}' is missing..."
	    return 1
	fi
	
	((offset++))
    done
    
    return 0
}

##
##
##
function PrepareInstallation() {
    ##
    ## Check for programs
    ##

    ## Wine
    CheckWineVersion
    ret=$?
    if [ $ret != 0 ]; then
	echo "Something is wrong with your Wine's installation, exiting."
	exit 1
    else
	echo "Your Wine installation looks fine, let's continue..."
    fi


    ## Winetricks
    CheckWinetricks
    ret=$?
    if [ $ret != 0 ]; then
	echo "Something is wrong with winetricks, exiting."
	exit 1
    else
	echo "Your Wine installation looks fine, let's continue..."
    fi

    ## Check installation directory (will remove old installation, if any)
    if [ -e "$WINEPREFIX" -a -d "$WINEPREFIX" ]; then
	echo -ne "The current specific installation of Wine for OpenGD77 CPS ($WINEPREFIX) will be erased.\n  >> Please confirm (y/N): "
	read confirm

	case "$confirm" in
	    y|Y|o|O)
		## rm
		echo "Deleting old install from $WINEPREFIX..."
		rm -rf "$WINEPREFIX"
		;;
	    
	    *)
		return 1
		;;
	esac
    fi

    ## Install fresh wine
    echo -e "\n\nCreating a fresh wine configuration for the OpenGD77 CPS...\n\n"
    $WINETRICKS_BIN win7 --optout
    wineboot -u
    ret=$?
    if [ $ret != 0 ]; then
	echo "Something went wrong while initializing wine's installation...."
	return $ret
    fi

##    ## Serial ports
##    comPorts=("com10" "com11" "com12")
##    usbPorts=("ttyUSB0" "ttyUSB1" "ttyUSB2")
##    let fCount=${#comPorts[@]}
##    let n=0
##
##    ## wait until com ports are created by wine installation process
##    while ! [ -e "$WINEPREFIX/dosdevices/${comPorts[offset]}" ]; do
##	sleep 1s
##    done
##    
##    for n in $(seq 0 $(($fCount - 1))); do
##	echo "*** rm -f $WINEPREFIX/dosdevices/${comPorts[offset]}"
##	rm -f "$WINEPREFIX/dosdevices/${comPorts[offset]}"
##	echo "*** ln -sf /dev/${usbPorts[offset]} $WINEPREFIX/dosdevices/${comPorts[offset]}"
##	ln -sf "/dev/${usbPorts[offset]}" "$WINEPREFIX/dosdevices/${comPorts[offset]}"
##	((offset++))
##    done

    ## Install winetricks modules
    echo -e "\n\nInstalling .NET for the OpenGD77 CPS (may takes some time, and requires few confirmations in the GUI)...\n\n" 
    InstallWinetricksModules
    ret=$?

    return $ret
}

##
##
##
function DownloadAndInstall()
{
    dltools=("wget" \
		 "curl")
    dlcmdlines=("--progress=bar -O /tmp/OpenGD77CPSInstaller.exe $CPS_URL" \
		    "--progress-bar -o /tmp/OpenGD77CPSInstaller.exe $CPS_URL")
    let dloffset=-1
    let offset=0
    
    let fCount=${#dltools[@]}
    let n=0
    for n in $(seq 0 $(($fCount - 1))); do
        if [ -x "$(command -v ${dltools[offset]})" ]; then
	    dloffset=$offset
	    break;
        fi
	((offset++))
    done

    if [ $dloffset -eq -1 ]; then
	echo "You need 'wget' or 'curl' to download the OpenGD77 CPS."
	return 1
    fi

    ## Download the installer
    ${dltools[dloffset]} ${dlcmdlines[dloffset]}
    ret=$?
    if [ $ret != 0 ]; then
	echo "Something went wrong while downloading $CPS_URL using ${dltools[dloffset]}, aborting."
	exit 1
    fi

    ## Execute installer
    echo "executing $WINE_BIN /tmp/OpenGD77CPSInstaller.exe"
    $WINE_BIN "/tmp/OpenGD77CPSInstaller.exe"
    ret=$?
    echo "Exit code: $ret"
    
    ## Remove temporary file
    rm -f "/tmp/OpenGD77CPSInstaller.exe"
    
    exit $ret
}


##
## Main
##

##
## Parse the command line
##
if [ $# -eq 0 ]; then

    if [ -f "$CPS_BIN" -a -x "$CPS_BIN" ]; then
	echo -ne "\n\nLaunching '$CPS_BIN'...\n\n"
	$WINE_BIN "$CPS_BIN"
    else
	Usage
    fi
else
    while [[ $# -gt 0 ]]; do
	key="$1"
	
	case $key in
            "-h" | "--help")
		Usage
		break
		;;
	    
            "-p" | "--prepare")
		PrepareInstallation
		break
		shift
		;;

	    "-d" | "--download")
		DownloadAndInstall
		ret=$?
		if [ $ret != 0 ]; then
		    echo "Something went wrong while downloading or installating the OpenGD77 CPS, exiting."
		    exit 1
		fi
		break
		;;
	    
            "-i" | "--install")
		PrepareInstallation
		ret=$?
		if [ $ret != 0 ]; then
		    echo "Something went wrong while preparing wine's installation, aborting."
		    exit 1
		else
		    DownloadAndInstall
		    ret=$?
		    if [ $ret != 0 ]; then
			echo "Something went wrong while downloading or installating the OpenGD77 CPS, exiting."
			exit 1
		    fi
		    break
		fi
		shift
		;;

            "-c" | "--check")
		## Wine
		CheckWineVersion
		ret=$?
		if [ $ret == 0 ]; then
		    echo "Your Wine installation looks fine, let's continue..."
		else
		    echo "Something is wrong with your Wine's installation, this is bad."
		fi

		## Wine directory installation
		if [ -e "$WINEPREFIX" -a -d "$WINEPREFIX" ]; then
		    echo "Custom directory for wine installation exists, this is good."
		else
		    echo "Custom directory for wine installation DO NOT exists, this is bad."
		fi

		## Winetricks
		CheckWinetricks
		ret=$?
		if [ $ret == 0 ]; then
		    echo "winetricks is installed, this is good."
		    ## Winetricks modules
		    CheckWinetricksModules
		    ret=$?
		    if [ $ret == 0 ]; then
			echo "Modules ${WINETRICKS_MODULES[@]} are installed, this is good."
		    else
			echo "Some winetricks modules are missing, this is bad. Restart this script with --install-modules argument to fix this situation."
		    fi
		fi
		    
		echo "Checkings are done now."
		
		break
		shift
		;;

	    "-m" | "--install-modules")
		## Winetricks
		CheckWinetricks
		ret=$?
		if [ $ret == 0 ]; then
		    echo "winetricks is installed, this is good."
		    ## Winetricks package
		    InstallWinetricksModules
		    ret=$?
		    if [ $ret == 0 ]; then
			echo "Modules ${WINETRICKS_MODULES[@]} are installed, this is good."
		    else
			echo "Some winetricks modules are missing, this is bad. Restart this script with --install-modules argument to fix this situation."
		    fi
		fi

		shift
		;;

	    "-w" | "--winetricks")
		## Winetricks
		CheckWinetricks
		ret=$?
		if [ $ret == 0 ]; then
		    $WINETRICKS_BIN
		    exit $?
		fi

		break
		shift
		;;
	    
            *)
		## Start wine with arg
		if [ -f "$key" ]; then
		    echo -ne "\n\nStarting the OpenGD77 CPS using $key\n\n"
		    $WINE_BIN "$key"
		    ret=$?
		    echo "Exit code: $ret"
		    exit $ret
		else
		    echo "Unknown argument: $key"
		    exit 1
		fi
		;;
	esac    
    done
fi

exit 0
